// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// under GPL or new BSD license

#include <kos.h>

#include <GL/gl.h>
#include <GL/glu.h>

extern "C" {
#include "all.h"
#include "glwrap.h"
}

#include "entitymd2.h"
#include "entity3ds.h"
#include "utils/3ds.h"

KOS_INIT_FLAGS(INIT_DEFAULT);
#ifndef NOROMDISK
extern uint8 romdisk[];
KOS_INIT_ROMDISK(romdisk);
#endif

int	zclip = false;
int skydome_txr;


t3DModel g_3DModel;	// This holds the 3DS Model info that we load in

#define FPS 60

void set_projection (void)
{
    float	zNear = 1;
    float	zFar = 4000;
    float	aspectratio = 640.0f / 480.0f;
    float	field_of_view = 45; // degrees


    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    gluPerspective (field_of_view, aspectratio, zNear, zFar);

}

void enable_culling(){
    glEnable(GL_CULL_FACE);
};
void disable_culling(){ glDisable(GL_CULL_FACE);};


int main(int argc, char **argv) {

    uint8	c;
    maple_device_t * caddr;
    cont_state_t * cont;
    pvr_stats_t stats;

    float	z = 0;
    float y=0;

    float tx=0, ty=0, tz=0;
    float rx=0.0f, ry=0.0f, rz=0.0f;

    int	a_pressed = false;

    GLfloat camera[3]={0,0,0};

    GLfloat look[3]={0,0,-500};

    GLfloat up[3]={0,1,0};



    /* Normal init */
    pvr_init_defaults();


    //init KGL and set the default viewport stuff
    glKosInit();


    //init texture.c
    texture_init();

    set_projection();

    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity();

    glEnable(GL_TEXTURE_2D);

    glShadeModel(GL_SMOOTH);

    glDisable(GL_KOS_AUTO_UV);

    // Enable back face culling
    enable_culling();


    /* Expect CW verts */
    glFrontFace(GL_CW);

    //grab controller 1
    c = maple_first_controller();



    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_FILTER, GL_FILTER_BILINEAR);

    //3DS loading


    //loading some 3ds files

    CLoad3DS g_Load3ds;	// This is 3DS class.  This should go in a good model class.
    g_Load3ds.Import3DS(&g_3DModel,"/rd/ship.3ds");			// Load our .3DS file into our model structure

    // Depending on how many textures we found, load each one (Assuming .JPG)
    // Go through all the materials

    for(int i = 0; i < g_3DModel.numOfMaterials; i++)
    { int id=-1;
        // Check to see if there is a file name to load in this material
        if(strlen(g_3DModel.pMaterials[i].strFile) > 0)
        {
            //printf("Loading 3ds material %s",g_3Dmodel.pMaterials[i].strFile);

            //HACK
            g_3DModel.g_Texture[i]=auto_load_texture("/rd/ship.jpg",JPG);

        }else
            g_3DModel.g_Texture[i]=-1;

        // Set the texture ID for this material
        g_3DModel.pMaterials[i].texureId = i;
    }

    //create the 3DS entity
    ENTITY_3DS *ent_space = new ENTITY_3DS();
    ent_space->SetModel(&g_3DModel);


    //MD2 loading
    ENTITY_MD2 *ent_zombie = new ENTITY_MD2();

    ENTITY_MD2 *ent = new ENTITY_MD2();
    ENTITY_MD2 *ent_w = new ENTITY_MD2();

    ent->Load("/rd/Spiff.md2","/rd/Spiff.jpg");
    ent_w->Load("/rd/chaingun.md2","/rd/chaingun.jpg");

    //load some models from the CD
    //    ent->Load("/cd/romdisk/Spiff.md2","/cd/romdisk/Spiff.jpg");
    //    ent_w->Load("/cd/romdisk/chaingun.md2","/cd/romdisk/chaingun.jpg");

    //  ent_zombie->Load("/cd/romdisk/zombie.md2","/cd/romdisk/Zombie1.jpg");


    //set his position

    ent_zombie->SetPosition(-0.8,0,-8);

    ent_zombie->SetBlendCount(FPS/3);

    //set his first and last animation frame

    ent_zombie->SetAnimStart(0);
    ent_zombie->SetAnimEnd(10);

    //set his position

    ent->SetPosition(0,0,-5);

    ent->SetBlendCount(FPS/4);

    //set his first and last animation frame

    ent->SetAnimStart(0);
    ent->SetAnimEnd(172);

    ent_w->SetPosition(0,0,-5);

    ent_w->SetBlendCount(FPS/4);

    //set his first and last animation frame

    ent_w->SetAnimStart(0);
    ent_w->SetAnimEnd(172);


    //engine loop starts here
    while(1) {

        z=y=0;

        /* Check controller/key status */
        if ((caddr = maple_enum_type(0, MAPLE_FUNC_CONTROLLER)) == NULL)
            break;

        cont = (cont_state_t *)maple_dev_status(caddr);
        if ((cont->buttons & CONT_START))
        {
            //frees the memory .just a test if DC hangs
            delete ent;
            delete ent_w;
            delete ent_zombie;
            delete ent_space;
            break;
        }
        if ((cont->buttons & CONT_DPAD_LEFT))
        { y = 0.01f;

            tx += fsin(rx*F_PI/180.0f);
        }
        if ((cont->buttons & CONT_DPAD_RIGHT))
        {  y = -0.01f;

            tx -= fsin(rx*F_PI/180.0f);
        }
        if ((cont->buttons & CONT_DPAD_DOWN))
        {
            z = -0.05f;
            tz -= fcos(ry*F_PI/180.0f);

        }
        if ((cont->buttons & CONT_DPAD_UP))
        {  z= 0.05f;
            tz += fcos(ry*F_PI/180.0f);
        }
        if ((cont->buttons & CONT_A) && !a_pressed) {
            a_pressed = true;
            zclip = !zclip;
        }
        if (!(cont->buttons & CONT_A))
            a_pressed = false;

        if ((cont->buttons & CONT_Y)) {
            static int count=1;
            char  buffer[256];
            sprintf(buffer,"/pc/home/shoots/shot%..ppm", count++);
            vid_screen_shot(buffer);


        }

        rx+=((float)cont->joyy)/100.0f;
        ry+=((float)cont->joyx)/100.0f;



        //tell KGL we're rendering a new frame
        glKosBeginFrame();

        //   set the camera position
        glwLookAt(camera, look,up);

        glColor4f(1,1,1,1);



#if 1
        static int rotater=0;

        enable_culling();

        ent_zombie->Delta_y=ent_w->Delta_y=ent->Delta_y=ry;
        ent_zombie->Delta_x=ent_w->Delta_x=ent->Delta_x=rx;

        ent_zombie->Movement_y=ent_w->Movement_y=ent->Movement_y=y;
        ent_zombie->Movement_z=ent_w->Movement_z=ent->Movement_z=z;

        glPushMatrix();
        //update position und rotation
        ent->Update();
        //apply  KGL matrixstack (MODELVIEW!)
        ent->Apply();
        ent->Draw();
        glPopMatrix();

        glPushMatrix();
        //Weapon model
        //update position und rotation
        ent_w->Update();
        //apply  KGL matrixstack (MODELVIEW!)
        ent_w->Apply();
        ent_w->Draw();

        disable_culling();


        if(++rotater>360)rotater=0;


        glRotatef((GLfloat)rotater,0,1,0);
      	glRotatef((GLfloat)rotater,1,0,0);

        glTranslatef(-1, 0, 0.55);

        glScalef(1.0f/10, 1.0f/10, 1.0f/10);


        ent_space->Draw();

        glPopMatrix();

#if 0
        glPushMatrix();
        ent_zombie->Update();
        ent_zombie->Apply();

        //HACK: model rotating to the front
        //   glRotatef(-90, 0.0f, 1.0f, 0.0f);
        //  ent_zombie->Draw();

        disable_culling();

        rotater=rotater+1.0f;
        if(rotater>360)rotater=0;

        //HACK: ent_space uses the Matrix from ent_zombie via Update() and Apply()
        //	just to show that you do not need a complete entity class
        // for each object!
        glTranslatef(0, 0, 0.05);
        glRotatef(rotater,0,1,0);
        glScalef(1.0f/10, 1.0f/10, 1.0f/10);
        ent_space->Draw();

        glPopMatrix();
  #endif
 #endif



        //  //tell KOS our opaque frame is done
        //         glKosFinishList();
        // 		glEnable(GL_BLEND);
        // put your blending stuff here->

        // <-
        //tell KOS our frame is done
        glKosFinishFrame();
        glDisable(GL_BLEND);

#if 0
        pvr_get_stats(&stats);
        if((stats.frame_count%620)==0){


            printf("Frame Count: %d, last_time: %f, frame rate: %f fps\n",
                   stats.frame_count, stats.frame_last_time, stats.frame_rate);

            printf("Camera position: %f, %f, %f\n",
                   camera[0],   camera[1],   camera[2]);

        }
        //end of engine loop
#endif
    }
    //end of program
    return 0;
}

