// VECTOR Class
// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// under GPL or new BSD license

#ifndef VectorH
#define VectorH

#include <dc/fmath.h>
#include <GL/gl.h>
#include "../utils.h"

#define pi F_PI
#define F_PI2 (2.0f*F_PI)
#define radian  (F_PI / 180.0f)

#define DEG2RAD(a)	(F_PI/180.0f*(a))
#define RAD2DEG(a)	(180.0f/F_PI*(a))

class VECTOR
{
public:
    VECTOR(GLfloat sx = 0, GLfloat sy = 0, GLfloat sz = 0);
    ~VECTOR();


	const bool operator == (const VECTOR& Vector) const
	{
		return (x == Vector.x && y == Vector.y && z == Vector.z);
	}
	const bool operator != (const VECTOR& Vector) const
	{
		return !(*this == Vector);
	}

    // + -
    VECTOR operator + (const VECTOR &V) const
    {
        return VECTOR(x + V.x, y + V.y, z + V.z);
    }

    VECTOR &operator += (const VECTOR &V)
    {
        x += V.x; y += V.y; z += V.z;
        return *this;
    }
    VECTOR operator - () const
    {
        return VECTOR(-x, -y, -z);
    }
    VECTOR operator - (const VECTOR &V) const
    {
        return VECTOR(x - V.x, y - V.y, z - V.z);
    }
    VECTOR &operator -= (const VECTOR &V)
    {
        x -= V.x; y -= V.y; z -= V.z;
        return *this;
    }
//     VECTOR &operator * (const VECTOR &V)//////////////
//     {
//         x *= V.x; y *= V.y; z *= V.z;
//         return *this;
//     }////////////////////////////////////////////////////
    VECTOR operator * (float f) const
    {
        return VECTOR(x * f, y * f, z * f);
    }
    VECTOR &operator *= (float f)
    {
        x *= f; y *= f; z *= f;
        return *this;
    }
    // cross product
    VECTOR operator^(const VECTOR& V) const
    {
        return VECTOR(y*V.z - z*V.y, z*V.x - x*V.z, x*V.y - y*V.x);
    }
    // dot product
    float operator | (const VECTOR &V) const
    {

        return x*V.x + y*V.y + z*V.z;
        // NormOfVector | Vector = LengthOfVector
    }

    GLvoid Reset();
    GLvoid Set(GLfloat sx, GLfloat sy, GLfloat sz) {x = sx, y = sy, z = sz;}
    GLvoid CrossVector(VECTOR vect);
    GLfloat DotProduct(VECTOR vect);

    GLfloat inline Magnitude()
    {
	    return mat_veq(x,y,z);
       // return fsqrt(x*x + y*y + z*z);//FIXME :durch SH4 ersetzen
    }
   VECTOR inline &Normalize()
    {
        GLfloat OneDivMagnitude = 1.0f/Magnitude();
        x *= OneDivMagnitude;
        y *= OneDivMagnitude;
        z *= OneDivMagnitude;
        return *this;
    }
    VECTOR inline GetNormalized()
    {
        return VECTOR(*this).Normalize();
    }

    GLfloat x;
    GLfloat y;
    GLfloat z;
};

#endif

