
// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// based on code by Jimbo
// under GPL or new BSD license

#include <GL/gl.h>
#include <kos.h>

#include "texture.h"
#include "utils.h"


#define TRUE 1
#define FALSE 0

#define MAX_TEXTURES 128

GLint texture_ram[MAX_TEXTURES];
struct texture textures[MAX_TEXTURES];

int max_pos=-1;

void texture_init() {
    int i;
    glGenTextures(MAX_TEXTURES,&texture_ram[0]);
    for(i=0;i<MAX_TEXTURES;i++){

        textures[i].txr=NULL;
    }
}

GLint next_tex() {
    static int pos=0;
    GLint ret=texture_ram[pos];
    pos++;
    return ret;
}

// returns the next tex_position or -1 if error!
int next_tex_id() {
    int index=0;

    //find any free text Positions
    while(textures[index].txr!=NULL)
    {
        index++;
        if(index>MAX_TEXTURES) {assert(0);return -1;}
    }

    return index;


}

struct texture *get_tex(int index) {
    return &textures[index];
}

int find_tex(char *filename) {
    int index=0;
    for(index=0;index<=max_pos;index++) {

        if (textures[index].txr!=NULL)
            if(!strcmp(textures[index].filename,filename)) {

                //	DBG("found texture %s\n",textures[index].filename);

                return index;
            }
    }
    return -1;
}

void switch_tex(int pos) {

    //  printf("Switching to texture %i (%i x %i)\n",pos,textures[pos].xsize,textures[pos].ysize);

    glBindTexture(GL_TEXTURE_2D, textures[pos].txr);
}


void delete_tex(int pos) {

	if(pos<0) return;

    glDeleteTextures(1,&textures[pos].txr);
    textures[pos].txr=NULL;

}

void pset(struct texture tex, int x, int y, uint16 col) {
    tex.pixbuf[x+y*(tex.xsize)]=col;
}

uint16 color_fg(int color, int alpha) {
    int r,g,b,a=alpha/*=0x0f*/;
    //color palette based on VICE's default.vpl
    switch(color) {
    case 0:
        r = 0x10; g = 0x10; b = 0x10;
        break;
    case 1:
        r = 0xFF; g = 0xFF; b = 0xFF;
        break;
    case 2:
        r = 0xE0; g = 0x40; b = 0x40;
        break;
    case 3:
        r = 0x60; g = 0xFF; b = 0xFF;
        break;
    case 4:
        r = 0xE0; g = 0x60; b = 0xE0;
        break;
    case 5:
        r = 0x40; g = 0xE0; b = 0x40;
        break;
    case 6:
        r = 0x40; g = 0x40; b = 0xE0;
        break;
    case 7:
        r = 0xFF; g = 0xFF; b = 0x40;
        break;
    case 8:
        r = 0xE0; g = 0xA0; b = 0x40;
        break;
    case 9:
        r = 0x9C; g = 0x74; b = 0x48;
        break;
    case 10:
        r = 0xFF; g = 0xA0; b = 0xA0;
        break;
    case 11:
        r = 0x54; g = 0x54; b = 0x54;
        break;
    case 12:
        r = 0x88; g = 0x88; b = 0x88;
        break;
    case 13:
        r = 0xA0; g = 0xFF; b = 0xA0;
        break;
    case 14:
        r = 0xA0; g = 0xA0; b = 0xFF;
        break;
    case 15:
        r = 0xC0; g = 0xC0; b = 0xC0;
        break;
    default: //transparent
        r = 0; g = 0; b = 0; a=0x00;
        break;
    }
#ifdef WIN32
    return  (((r >> 3) & 0x1f) << 11)
            | (((g >> 2) & 0x3f) << 5)
            | (((b >> 3) & 0x1f) << 0);
#else
    return (a << 12) |
           ((r >> 1) << 8) |
           ((g >> 2) << 4) |
           ((b >> 1) << 0);
#endif
}


//jpeg loader
void load_texture(char *filename,int pos,int type) {

    int size;

    textures[pos].txr=next_tex();

    if(pos>max_pos) { max_pos=pos; }   strcpy(textures[pos].filename,filename);

    //     if(type==PCX) pcx_to_txr(filename,&textures[pos].txraddr);
    //     else if(type==JPG)
    jpeg_to_txr(filename, &textures[pos].txraddr,0,1,1,&size);
    //  else assert(0);


    textures[pos].xsize=size; textures[pos].ysize=size;

    glBindTexture(GL_TEXTURE_2D, textures[pos].txr);

    glKosTex2D(GL_RGB565_TWID, textures[pos].xsize,
               textures[pos].ysize,textures[pos].txraddr);


}


//jpeg loader
int auto_load_texture(char *filename,int type) {

    int size;

    int pos=next_tex_id();

    textures[pos].txr=next_tex();

    if(pos>max_pos) { max_pos=pos; }   strcpy(textures[pos].filename,filename);

    //     if(type==PCX) pcx_to_txr(filename,&textures[pos].txraddr);
    //     else if(type==JPG)
    jpeg_to_txr(filename, &textures[pos].txraddr,0,1,1,&size);
    // else assert(0);
    textures[pos].xsize=size; textures[pos].ysize=size;

    glBindTexture(GL_TEXTURE_2D, textures[pos].txr);

    glKosTex2D(GL_RGB565_TWID, textures[pos].xsize,
               textures[pos].ysize,textures[pos].txraddr);

    return pos;
}


